function y = randn_trunc_zero( mu, sigma, doA, nSamples)
%  Draw y from 1D normal distribution
%    truncated so output falls either above or below zero
%    y ~  N( mu, sigma)   restricted s.t.  y <= 0  if doA = 0
%                                          y  > 0  if doA = 1
% INPUTS
%   mu : scalar mean of the target Normal distribution
%   sigma: scalar standard deviation of target Normal distribution
%   doA  : binary indicator for whether to have output ABOVE zero (doA=1)
%                                             or  BELOW zero (doA=0)
%   nSamples  : number of samples to draw from this distribution
% OUTPUT
%   y  :  Nx1 vector of indep. draws from truncated normal
% NOTES
%   This routine is useful in probit regression models, which
%    assume the following generative model for data items i = 1,2, ... N
%    observed to have both *feature vectors* x_i = D-dim col vector
%                      and  *binary labels*  t_i \in {0, 1}
%    Pr( t_i = 1 | x_i ) = Bernoulli(  Probit( w^T x_i ) )
%    which is equivalent to an auxiliary variable model:
%        y_i ~ Normal( w^T x_i, 1 )
%        t_i = 1 iff y_i > 0
%            = 0 otherwise
%   We can see that given observed x_i,t_i pairs, sampling the hidden
%       y_i terms with this function can help us learn the weights "w"

if nargin < 4
    nSamples = 1;
end

if ( doA == 1)
     y = randn_trunc_below( mu, sigma, 0, nSamples );    
else    
     y = randn_trunc_above( mu, sigma, 0, nSamples );    
end

end
